from datetime import datetime
from pathlib import Path

from flask_sqlalchemy import SQLAlchemy


db_dir = Path(__file__).parent
db_file = db_dir / Path('funt.sqlite')
db = SQLAlchemy()


class Exam(db.Model):
    __tablename__ = 'exam'
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String, nullable=False)
    grade = db.Column(db.Integer, nullable=False)
    date_created = db.Column(db.DateTime, default=datetime.utcnow())

    def __repr__(self):
        return f'<Exam {self.id}>'



from flask import render_template, request, redirect, Blueprint
import sqlalchemy

from .model import Exam, db
from .tools import mean_function


bp = Blueprint('views', __name__)


@bp.route("/", methods=['POST', 'GET'])
def index():
    if request.method == 'POST':
        exam_name = request.form['courseName']
        exam_grade = request.form['examMark']
        new_exam = Exam(name=exam_name, grade=exam_grade)
        try:
            db.session.add(new_exam)
            db.session.commit()
            db.session.close()
            return redirect('/')
        except sqlalchemy.exc.SQLAlchemyError:
            return 'There was an issue adding your exam'
    else:
        exams = db.session.query(Exam).order_by(Exam.date_created).all()
        return render_template("index.html", exams=exams)


@bp.route("/grades-mean", methods=['GET'])
def mean():
    exams = db.session.query(Exam).order_by(Exam.date_created).all()
    grades = []
    for e in exams:
        grades.append(int(e.grade))
    grades_mean = round(mean_function(grades), 2)
    return render_template("mean.html", mean=grades_mean)


@bp.route("/clear-db", methods=['GET'])
def clear_db():
    db.drop_all()
    db.create_all()
    return redirect('/')



# Utility functions


def simple_function(test):
    return test + 1


def mean_function(test):
    count = 0
    total = 0
    for val in test:
        total += val
        count += 1
    return total / count


def max_function(test):
    if not test:
        return -1

    max_value = 0
    for val in test:
        if val > max_value:
            max_value = val
    return max_value


def min_function(test):
    if not test:
        return -1

    min_value = test[0]
    for val in test[1:]:
        if val < min_value:
            min_value = val
    return min_value



from flask import Flask

from . import views
from .model import db, db_file


def create_app():
    app = Flask(__name__)

    app.register_blueprint(views.bp)

    return app


def init_db(app):
    app.config["SQLALCHEMY_DATABASE_URI"] = f"sqlite:///{db_file}"
    db.init_app(app)

    with app.app_context():
        db.drop_all()
        db.create_all()
